#!/usr/local/bin/perl

#Web Gallery Professional → freo ログ変換ツール
#
#Copyright(C) 2002-2010 Knight, All rights reserved.
#Mail ... info at favorite-labo dot org
#Home ... http://freo.jp/

package main;

use strict;
use CGI::Carp qw(fatalsToBrowser);
use lib qw(./lib);

#――――― 設定項目開始 ―――――――――――――――――

#出力先ファイル
my $output_file = 'gallery.sql';

#データベースの種類(sqlite もしくは mysql)
my $database_type = 'sqlite';

#データベースのテーブル名のプレフィックス
my $database_prefix = 'freo_';

#ユーザー名の置換
my %users = (
	'admin' => 'example',
);

#テキストの置換
my @texts = (
	['http://www.example.com/gallerypro/gallery.cgi?mode=image&amp;upfile=', 'http://www.example.com/freo/files/medias/gallery/'],
	['http://www.example.com/gallerypro/data/upfile/',                       'http://www.example.com/freo/files/medias/gallery/'],
	['http://www.example.com/gallerypro/data/thumbnail/',                    'http://www.example.com/freo/files/media_thumbnails/gallery/'],
	['http://www.example.com/gallerypro/',                                   'http://www.example.com/freo/'],
);

#分類の置換
my %fields = (
	'未分類'                 => '',
	'雑記'                   => 'note',
	'雑記::日々の事'         => 'note/daily',
	'雑記::サイトの事'       => 'note/site',
	'雑記::仕事の事'         => 'note/work',
	'プログラム'             => 'program',
	'プログラム::Perl'       => 'program/perl',
	'プログラム::PHP'        => 'program/php',
	'プログラム::JavaScript' => 'program/js',
	'プログラム::C/C++'      => 'program/c',
	'プログラム::その他'     => 'program/other',
);

#――――― 設定項目終了 ―――――――――――――――――

#モジュール読み込み
use base qw(webliberty::Basis);
use webliberty::Parser;
use webliberty::Configure;
use webliberty::App::Init;
use webliberty::App::Gallery;

#インスタンス作成
my $basis_ins   = new webliberty::Basis;
my $init_ins    = new webliberty::App::Init;
my $parser_ins  = new webliberty::Parser;
my $config_ins  = new webliberty::Configure($init_ins->get_init('data_config'));
my $gallery_ins = new webliberty::App::Gallery($init_ins->get_init, $config_ins->get_config, $parser_ins->get_query);

#出力開始
open(OUTPUT, ">$output_file") or $basis_ins->error("Write Error : $output_file");

#記事
opendir(DIR, $init_ins->get_init('data_gallery_dir')) or $basis_ins->error("Read Error");
my @dir = sort { $b <=> $a } readdir(DIR);
closedir(DIR);

my $data_ext       = $init_ins->get_init('data_ext');
my $gallery_buffer = '';
my $field_buffer   = '';

foreach my $entry (@dir) {
	if ($entry !~ /^\d\d\d\d\d\d\.$data_ext$/) {
		next;
	}

	open(FH, $init_ins->get_init('data_gallery_dir') . $entry) or $basis_ins->error("Read Error : $entry");
	while (<FH>) {
		chomp;
		my($no, $id, $stat, $break, $view, $comt, $tb, $field, $date, $name, $subj, $text, $price, $icon, $image, $file, $host) = split(/\t/);

		if (!$stat) {
			next;
		}

		$text =~ s/-----/\n<!-- pagebreak -->\n/g;

		my %gallery = $gallery_ins->gallery_article($no, $id, $stat, $break, $view, $comt, $tb, $field, $date, $name, $subj, $text, $price, '', '', $file, '');

		my $article_no = $gallery{'ARTICLE_NO'};

		my $article_user = '\'' . ($users{$gallery{'ARTICLE_USER'}} ? $users{$gallery{'ARTICLE_USER'}} : $gallery{'ARTICLE_USER'}) . '\'';

		my $article_date = '\'' . $gallery{'ARTICLE_DATE'} . ':00\'';

		my $article_comt = '\'' . ($comt ? 'open' : 'closed') . '\'';

		my $article_tb = '\'' . ($tb ? 'open' : 'closed') . '\'';

		my $article_id = $gallery{'ARTICLE_ID'};
		if ($article_id eq '') {
			$article_id = 'NULL';
		} else {
			$article_id = "'$article_id'";
		}

		my $article_subj = $gallery{'ARTICLE_SUBJ'};
		if ($database_type eq 'mysql') {
			$article_subj =~ s/'/\\'/g;
		} else {
			$article_subj =~ s/'/''/g;
		}
		$article_subj = '\'' . $article_subj . '\'';

		my $article_tagging = $gallery{'ARTICLE_Tagging'};
		if ($article_tagging eq '') {
			$article_tagging = 'NULL';
		} else {
			$article_tagging =~ s/<[^>]*>//g;
			$article_tagging =~ s/ /,/g;
			$article_tagging = '\'' . $article_tagging . '\'';
		}

		my $article_text = ($gallery{'ARTICLE_FILES'} ? '<p>' . $gallery{'ARTICLE_FILES'} . '</p>' : '') . $gallery{'ARTICLE_TEXT'};
		foreach my $text (@texts) {
			my $from = quotemeta($text->[0]);
			my $to   = $text->[1];

			$article_text =~ s/$from/$to/g;
		}
		if ($database_type eq 'mysql') {
			$article_text =~ s/'/\\'/g;
		} else {
			$article_text =~ s/'/''/g;
		}
		$article_text = '\'' . $article_text . '\'';

		$gallery_buffer = "INSERT INTO " . $database_prefix . "entries VALUES($article_no, $article_user, $article_date, $article_date, 'yes', NULL, NULL, 'publish', 'publish', $article_comt, $article_tb, $article_id, $article_subj, $article_tagging, $article_date, NULL, NULL, NULL, NULL, $article_text);\n$gallery_buffer";

		if ($fields{$gallery{'ARTICLE_FIELD'}} ne '') {
			my $article_field = $fields{$gallery{'ARTICLE_FIELD'}};
			$article_field = '\'' . $article_field . '\'';

			$field_buffer .= "INSERT INTO " . $database_prefix . "category_sets VALUES($article_field, $article_no);\n";
		}
	}
	close(FH);
}

print OUTPUT "DELETE FROM " . $database_prefix . "entries;\n";
if ($database_type eq 'mysql') {
	print OUTPUT "ALTER TABLE " . $database_prefix . "entries AUTO_INCREMENT = 0;\n";
}
print OUTPUT $gallery_buffer;

#カテゴリー
print OUTPUT "DELETE FROM " . $database_prefix . "category_sets;\n";
print OUTPUT $field_buffer;

#コメント
print OUTPUT "DELETE FROM " . $database_prefix . "comments;\n";
if ($database_type eq 'mysql') {
	print OUTPUT "ALTER TABLE " . $database_prefix . "comments AUTO_INCREMENT = 0;\n";
}

open(FH, $init_ins->get_init('data_comt_index')) or $basis_ins->error("Read Error : " . $init_ins->get_init('data_comt_index'));
my @index = reverse(<FH>);
close(FH);

foreach my $index (@index) {
	my($no_index, $pno_index) = split(/\t/, $index);

	open(FH, $init_ins->get_init('data_comt_dir') . $pno_index . '.' . $init_ins->get_init('data_ext')) or $basis_ins->error("Read Error : $pno_index");
	while (<FH>) {
		chomp;
		my($no, $pno, $stat, $date, $name, $mail, $url, $subj, $text, $color, $icon, $file, $rank, $pwd, $host) = split(/\t/);

		if ($no_index == $no) {
			my %comment = $gallery_ins->comment_article($no, $pno, $stat, $date, $name, $mail, $url, $subj, $text, '', '', '', '', '', '');

			my $article_no = $comment{'ARTICLE_NO'};

			my $article_pno = $comment{'ARTICLE_PNO'};

			my $article_date = '\'' . $comment{'ARTICLE_DATE'} . ':00\'';

			my $article_name = $comment{'ARTICLE_NAME'};
			if ($database_type eq 'mysql') {
				$article_name =~ s/'/\\'/g;
			} else {
				$article_name =~ s/'/''/g;
			}
			$article_name = '\'' . $article_name . '\'';

			my $article_mail = $comment{'ARTICLE_MAIL'};
			if ($article_mail eq '') {
				$article_mail = 'NULL';
			} else {
				if ($database_type eq 'mysql') {
					$article_mail =~ s/'/\\'/g;
				} else {
					$article_mail =~ s/'/''/g;
				}
				$article_mail = '\'' . $article_mail . '\'';
			}

			my $article_url = $comment{'ARTICLE_URL'};
			if ($article_url eq '') {
				$article_url = 'NULL';
			} else {
				if ($database_type eq 'mysql') {
					$article_url =~ s/'/\\'/g;
				} else {
					$article_url =~ s/'/''/g;
				}
				$article_url = '\'' . $article_url . '\'';
			}

			my $article_text = $comment{'ARTICLE_TEXT'};
			$article_text =~ s/<br \/>/\n/g;
			$article_text =~ s/<\/p>/\n\n/g;
			$article_text =~ s/<[^>]*>//g;
			$article_text =~ s/&quot;/"/g;
			$article_text =~ s/&lt;/</g;
			$article_text =~ s/&gt;/>/g;
			$article_text =~ s/&amp;/&/g;
			$article_text =~ s/^\s+//;
			$article_text =~ s/\s+$//;
			if ($database_type eq 'mysql') {
				$article_text =~ s/'/\\'/g;
			} else {
				$article_text =~ s/'/''/g;
			}
			$article_text = '\'' . $article_text . '\'';

			print OUTPUT "INSERT INTO " . $database_prefix . "comments VALUES($article_no, $article_pno, NULL, NULL, $article_date, $article_date, 'yes', NULL, $article_name, $article_mail, $article_url, '127.0.0.1', $article_text);\n";

			last;
		}
	}
	close(FH);
}

#トラックバック
print OUTPUT "DELETE FROM " . $database_prefix . "trackbacks;\n";
if ($database_type eq 'mysql') {
	print OUTPUT "ALTER TABLE " . $database_prefix . "trackbacks AUTO_INCREMENT = 0;\n";
}

open(FH, $init_ins->get_init('data_tb_index')) or $basis_ins->error("Read Error : " . $init_ins->get_init('data_tb_index'));
my @index = reverse(<FH>);
close(FH);

foreach my $index (@index) {
	my($no_index, $pno_index) = split(/\t/, $index);

	open(FH, $init_ins->get_init('data_tb_dir') . $pno_index . '.' . $init_ins->get_init('data_ext')) or $basis_ins->error("Read Error : $pno_index");
	while (<FH>) {
		chomp;
		my($no, $pno, $stat, $date, $blog, $title, $url, $excerpt) = split(/\t/);

		if ($no_index == $no) {
			my %trackback = $gallery_ins->trackback_article($no, $pno, $stat, $date, $blog, $title, $url, $excerpt);

			my $trackback_no = $trackback{'TRACKBACK_NO'};

			my $trackback_pno = $trackback{'TRACKBACK_PNO'};

			my $trackback_date = '\'' . $trackback{'TRACKBACK_DATE'} . ':00\'';

			my $trackback_blog = $trackback{'TRACKBACK_BLOG'};
			if ($database_type eq 'mysql') {
				$trackback_blog =~ s/'/\\'/g;
			} else {
				$trackback_blog =~ s/'/''/g;
			}
			$trackback_blog = '\'' . $trackback_blog . '\'';

			my $trackback_title = $trackback{'TRACKBACK_TITLE'};
			if ($database_type eq 'mysql') {
				$trackback_title =~ s/'/\\'/g;
			} else {
				$trackback_title =~ s/'/''/g;
			}
			$trackback_title = '\'' . $trackback_title . '\'';

			my $trackback_url = $trackback{'TRACKBACK_URL'};
			if ($database_type eq 'mysql') {
				$trackback_url =~ s/'/\\'/g;
			} else {
				$trackback_url =~ s/'/''/g;
			}
			$trackback_url = '\'' . $trackback_url . '\'';

			my $trackback_excerpt = $trackback{'TRACKBACK_EXCERPT'};
			$trackback_excerpt =~ s/<[^>]*>//g;
			$trackback_excerpt =~ s/<[^>]*>//g;
			$trackback_excerpt =~ s/&quot;/"/g;
			$trackback_excerpt =~ s/&lt;/</g;
			$trackback_excerpt =~ s/&gt;/>/g;
			$trackback_excerpt =~ s/&amp;/&/g;
			if ($database_type eq 'mysql') {
				$trackback_excerpt =~ s/'/\\'/g;
			} else {
				$trackback_excerpt =~ s/'/''/g;
			}
			$trackback_excerpt = '\'' . $trackback_excerpt . '\'';

			print OUTPUT "INSERT INTO " . $database_prefix . "trackbacks VALUES($trackback_no, $trackback_pno, NULL, $trackback_date, $trackback_date, 'yes', $trackback_blog, $trackback_url, '127.0.0.1', $trackback_title, $trackback_excerpt);\n";

			last;
		}
	}
	close(FH);
}

close(OUTPUT);

#完了画面
print $basis_ins->header;

print "<html>\n";
print "<head>\n";
print "<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\" />\n";
print "<title>データ変換</title>\n";
print "</head>\n";
print "<body>\n";
print "<p>データの変換が完了しました。</p>\n";
print "</body>\n";
print "</html>\n";

exit;
